function Iprocessed = nprprocess(inName,outName)

%========================================================================
% Non-photorealistic Rendering, Version 1.0
% Author: Yamini Nimmagadda
%
% This is an implementation of the non-photorealistic rendering algorithm
% for reducing image sizes by removing fine details. Please refer to
% to the following paper:
% 
% Yamini Nimmagadda, Yung-Hsiang Lu, Edward J. Delp, and David S. Ebert,
% "Non-photorealistic Rendering for Energy Conservation in Portable 
% Devices", Electronic Imaging (ISTSPIE), 2008.
%
% Kindly report any suggestions or corrections to ynimmaga@purdue.edu
%
%----------------------------------------------------------------------
%
% Input : inName: the name of the image to be rendered. The images 
%         should be uploaded in the directory containing this code.
%
% Output: outName: the name of the rendered image. The output image is 
%         stored in the same directory.
                                             

h = fspecial('gaussian', [3, 3], 0.5); % Gaussian filter for image smoothing
F = inName; 
I_ini = imread(F); % Read the intensity values of input image
I = I_ini;
F = outName;
T = strcat(F,'_or.JPG.smaller.jpg'); % Store the input image in JPEG
                                     % format for comparison of sizes
imwrite(I, T);

% Dividing the image into red, green and blue layers and filtering the 
% noise using Gaussian filter

I_red = I(:, :, 1);      
I_red = imfilter(I_red, h);

I_green = I(:, :, 2);
I_green = imfilter(I_green, h);

I_blue = I(:, :, 3);
I_blue = imfilter(I_blue, h);

% The function 'thresh' determines the intensity gradient threshold
% for all the three layers.

[rthresh, rmag] = thresh(I_red);
[gthresh, gmag] = thresh(I_green);
[bthresh, bmag] = thresh(I_blue);


th = [rthresh, gthresh, bthresh];
t = max(th);

% Make copies of the three layers to determine the edges

I_redt = I_red;
I_greent = I_green;
I_bluet = I_blue;

% Finding edges using the improved edge detector

A1= edge(I_redt, 'canny');
A2= edge(I_redt, 'sobel');
A3= edge(I_greent, 'canny');
A4= edge(I_greent, 'sobel');
A5= edge(I_bluet, 'canny');
A6= edge(I_bluet, 'sobel');

% The function 'superimpose' determines the gradients along
% all the four directions around a pixel and determines edges

D = superimpose(A2, A4);
B = superimpose(D, A6);
C = superimpose(A5, A1); 
B1 = superimpose(A5, A3);

% The parameter IGT for processign images is obtained by
% taking the maximum value among all the three layers

xmag = [rmag, gmag, bmag];
mmag = max(abs(xmag))       

% The function 'processlayer' segments each layer of the image

[I_red, rsegment] = processlayer(I_red, A1, t, abs(mmag));   
[I_green, gsegment] = processlayer(I_green, A3, t, abs(mmag)); 
[I_blue, bsegment] = processlayer(I_blue, A5, t, abs(mmag));
segmentrg = superimpose(rsegment, gsegment);
segment = superimpose(bsegment, segmentrg);

% The images are normalized using the 'normalize' function.

I_red = normalize(I_red, segment, rmag, 1);
progress = 33 % Displays progress of the normalization
I_green = normalize(I_green, segment, gmag, 1);
progress = 66 % Displays progress of the normalization
I_blue = normalize(I_blue, segment, bmag, 1);
progress = 99 % Displays progress of the normalization

I_red = imfilter(I_red, h);
I_green = imfilter(I_green, h);
I_blue = imfilter(I_blue, h);

% Modified red, green and blue layers
I_end(:, :, 1) = I_red;
I_end(:, :, 2) = I_green;
I_end(:, :, 3) = I_blue;

% Superimposed modified image
T = F;
imwrite(I_end, T); % Save the image in the parent directory.



