function [I, I_segment] = processlayer(I, B1, t, mag)

%========================================================================
% Non-photorealistic Rendering, Version 1.0
% Author: Yamini Nimmagadda
%
% This is an implementation of the non-photorealistic rendering algorithm
% for reducing image sizes by removing fine details. Please refer to
% to the following paper:
%
% Yamini Nimmagadda, Yung-Hsiang Lu, Edward J. Delp, and David S. Ebert,
% "Non-photorealistic Rendering for Energy Conservation in Portable
% Devices", Electronic Imaging (ISTSPIE), 2008.
%
% Kindly report any suggestions or corrections to ynimmaga@purdue.edu
%
%----------------------------------------------------------------------
%
% Input : (1) I: input image to be rendered
%         (2) B1: edge image determined using improved edge detector
%         (3) t: intensity gradient threshold
%         (4) mag: frequency gradient threshold
%
% Output: (1) I: input image copied back to output after segmentation
%         (2) I_segment: segmented image

% Determine the gradient of image I
[g_x, g_y] = gradient(double(I));


% Labeling the pixels
[m,n] = size(I);
k = 1;

for i = 1 : m
    for j = 1 : n
        I_segment(i, j) = k;
        k = k + 1;
    end
end

%Finding the gradient slope of all the pixels

for i = 1 : m
    for j = 1 : n
        if (g_x(i, j) == 0)
            grad_angle(i, j) = pi / 2;
        else
            grad_angle(i, j) = atan(g_y(i, j) / g_x(i, j));
        end
    end
end



% Classification of the gradient slopes into one of the four directions


for i = 1 : m
    for j = 1 : n
        if(grad_angle(i, j) <= pi / 8 & grad_angle(i, j) > -(pi / 8))
            grad_angle(i, j) = 0;
        elseif(grad_angle(i, j) <= 3 * (pi / 8) & grad_angle(i, j) > pi / 8)
            grad_angle(i, j) = pi / 4;
        elseif(grad_angle(i, j) <= -(pi / 8) & grad_angle(i, j) > - 3 * (pi / 8))
            grad_angle(i, j) = - pi / 4;
        elseif(grad_angle(i, j) <= - 3 * (pi / 8) & grad_angle(i, j) > 3 * (pi / 8))
            grad_angle(i, j) = pi / 2;
        end
    end
end


% If the neighbours are edges, make the pixel under consideration also
% an edge
B1_1 = B1;



for i = 2 : m - 1
    for j = 2 : n - 1
        if(B1_1(i, j) == 0)
            if (abs(grad_angle(i, j) - grad_angle(i - 1, j)) < pi/4 & B1(i - 1, j) == 1)
                B1_1(i, j) = 1;
            end
            if (abs(grad_angle(i, j) - grad_angle(i, j - 1)) < pi/4 & B1(i, j - 1) == 1)
                B1_1(i, j) = 1;
            end
            if (abs(grad_angle(i, j) - grad_angle(i - 1, j - 1)) < pi/4 & B1(i - 1, j - 1) == 1)
                B1_1(i, j) = 1;
            end
            if (abs(grad_angle(i, j) - grad_angle(i - 1, j + 1)) < pi/4 & B1(i - 1, j + 1) == 1)
                B1_1(i, j) = 1;
            end
        end
    end
end

% If the intensity difference between two pixels is greater than IGT and frequency
% difference is greater than frequency threshold, make the pixel under consideration 
% an edge

B1_3 = B1_1;

for i = 3 : m - 2
    for j = 3 : n - 2
        if (B1_1(i, j) == 1)
            if (grad_angle(i, j) == 0)
                if (abs(I(i - 1, j + 1) - I(i + 1, j + 1)) < mag)
                    B1_3(i, j + 1) = 1;
                end
                if (abs(I(i - 1, j - 1) - I(i + 1, j - 1)) < mag)
                    B1_3(i, j - 1) = 1;
                end
            elseif (grad_angle(i, j) == pi / 4)
                if (abs(I(i - 2, j) - I(i, j + 2)) < mag)
                    B1_3(i - 1, j + 1) = 1;
                end
                if (abs(I(i, j - 2) - I(i + 2, j)) < mag)
                    B1_3(i + 1, j - 1) = 1;
                end
            elseif (grad_angle(i, j) == pi / 2)
                if (abs(I(i - 1, j + 1) - I(i - 1, j - 1)) < mag)
                    B1_3(i - 1, j) = 1;
                end
                if (abs(I(i + 1, j - 1) - I(i + 1, j + 1)) < mag)
                    B1_3(i + 1, j) = 1;
                end
            elseif (grad_angle(i, j) == - pi / 4)
                if (abs(I(i, j + 2) - I(i + 2, j)) < mag)
                    B1_3(i + 1, j + 1) = 1;
                end
                if (abs(I(i - 2, j) - I(i, j - 2)) < mag)
                    B1_3(i - 1, j - 1) = 1;
                end
            end
        end
    end
end

for i = 3 : m - 2
    for j = 3 : n - 2
        if (B1_1(i, j) == 1)
            if (grad_angle(i, j) == 0)
                if (abs(I(i - 1, j + 1) - I(i + 1, j + 1)) < mag)
                    B1_3(i, j + 1) = 1;
                end
                if (abs(I(i - 1, j - 1) - I(i + 1, j - 1)) < mag)
                    B1_3(i, j - 1) = 1;
                end
            elseif (grad_angle(i, j) == pi / 4)
                if (abs(I(i - 2, j) - I(i, j + 2)) < mag)
                    B1_3(i - 1, j + 1) = 1;
                end
                if (abs(I(i, j - 2) - I(i + 2, j)) < mag)
                    B1_3(i + 1, j - 1) = 1;
                end
            elseif (grad_angle(i, j) == pi / 2)
                if (abs(I(i - 1, j + 1) - I(i - 1, j - 1)) < mag)
                    B1_3(i - 1, j) = 1;
                end
                if (abs(I(i + 1, j - 1) - I(i + 1, j + 1)) < mag)
                    B1_3(i + 1, j) = 1;
                end
            elseif (grad_angle(i, j) == - pi / 4)
                if (abs(I(i, j + 2) - I(i + 2, j)) < mag)
                    B1_3(i + 1, j + 1) = 1;
                end
                if (abs(I(i - 2, j) - I(i, j - 2)) < mag)
                    B1_3(i - 1, j - 1) = 1;
                end
            end
        end
    end
end


% Based on the gradient angle, make the pixel an edge depending on the
% intensity difference between pixels along the edge.

% Image segmentation

for i = 2 : m - 1
    for j = 2 : n - 1                           
        if (B1_3(i, j) == 0)
                     
            if ((abs(double(I(i, j)) - double(I(i - 1, j - 1))) < t)  && (B1_3(i - 1, j - 1) ~= 1)) % comparison to up left
                mi = min(I_segment(i, j), I_segment(i - 1, j - 1));
                I_segment(i, j) = mi;
                I_segment(i - 1, j - 1) = mi;
            end

            if ((abs(double(I(i, j)) - double(I(i - 1, j))) < t) && (B1_3(i - 1, j ) ~= 1))% comparison to up
                mi = min(I_segment(i, j), I_segment(i - 1, j));
                I_segment(i, j) = mi;
                I_segment(i - 1, j) = mi;
            end

            if ((abs(double(I(i, j)) - double(I(i - 1, j + 1))) < t) && (B1_3(i - 1, j + 1) ~= 1))% comparison to up right
                mi = min(I_segment(i, j), I_segment(i - 1, j + 1));
                I_segment(i, j) = mi;
                I_segment(i - 1, j + 1) = mi;
            end
           
            if ((abs(double(I(i, j)) - double(I(i, j - 1))) < t) && (B1_3(i, j - 1) ~= 1)) % comparison to the left
                mi = min(I_segment(i, j), I_segment(i, j - 1));
                I_segment(i, j) = mi;
                I_segment(i, j - 1) = mi;
            end

            if ((abs(double(I(i, j)) - double(I(i, j + 1))) < t) && (B1_3(i, j + 1) ~= 1))% comparison to right
                mi = min(I_segment(i, j), I_segment(i, j + 1));
                I_segment(i, j) = mi;
                I_segment(i, j + 1) = mi;
            end

            if ((abs(double(I(i, j)) - double(I(i + 1, j - 1))) < t) && (B1_3(i + 1, j - 1) ~= 1))% comparison to down left
                mi = min(I_segment(i, j), I_segment(i + 1, j - 1)); 
                I_segment(i, j) = mi;
                I_segment(i + 1, j - 1) = mi;
            end

            if ((abs(double(I(i, j)) - double(I(i + 1, j))) < t) && (B1_3(i + 1, j) ~= 1)) % comparison to down
                mi = min(I_segment(i, j), I_segment(i + 1, j));
                I_segment(i, j) = mi;
                I_segment(i + 1, j) = mi;
            end
                     
            if ((abs(double(I(i, j)) - double(I(i + 1, j + 1))) < t) && (B1_3(i + 1, j + 1) ~= 1))% comparison to down right
                mi = min(I_segment(i, j), I_segment(i + 1, j + 1));
                I_segment(i, j) = mi;
                I_segment(i + 1, j + 1) = mi;
            end            

        end
    end
end

B1 = B1_3;

